from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BasicSystemInformation")


@attr.s(auto_attribs=True)
class BasicSystemInformation:
    """ """

    bot_name: Union[Unset, str] = UNSET
    superadmin: Union[Unset, None, str] = UNSET
    org: Union[Unset, None, str] = UNSET
    org_id: Union[Unset, None, int] = UNSET
    bot_version: Union[Unset, str] = UNSET
    php_version: Union[Unset, str] = UNSET
    os: Union[Unset, str] = UNSET
    db_type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bot_name = self.bot_name
        superadmin = self.superadmin
        org = self.org
        org_id = self.org_id
        bot_version = self.bot_version
        php_version = self.php_version
        os = self.os
        db_type = self.db_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bot_name is not UNSET:
            field_dict["bot_name"] = bot_name
        if superadmin is not UNSET:
            field_dict["superadmin"] = superadmin
        if org is not UNSET:
            field_dict["org"] = org
        if org_id is not UNSET:
            field_dict["org_id"] = org_id
        if bot_version is not UNSET:
            field_dict["bot_version"] = bot_version
        if php_version is not UNSET:
            field_dict["php_version"] = php_version
        if os is not UNSET:
            field_dict["os"] = os
        if db_type is not UNSET:
            field_dict["db_type"] = db_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bot_name = d.pop("bot_name", UNSET)

        superadmin = d.pop("superadmin", UNSET)

        org = d.pop("org", UNSET)

        org_id = d.pop("org_id", UNSET)

        bot_version = d.pop("bot_version", UNSET)

        php_version = d.pop("php_version", UNSET)

        os = d.pop("os", UNSET)

        db_type = d.pop("db_type", UNSET)

        basic_system_information = cls(
            bot_name=bot_name,
            superadmin=superadmin,
            org=org,
            org_id=org_id,
            bot_version=bot_version,
            php_version=php_version,
            os=os,
            db_type=db_type,
        )

        basic_system_information.additional_properties = d
        return basic_system_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ModuleAccessLevel")


@attr.s(auto_attribs=True)
class ModuleAccessLevel:
    """ """

    name: Union[Unset, str] = UNSET
    value: Union[None, Unset, int, str] = UNSET
    numeric_value: Union[Unset, int] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        value: Union[None, Unset, int, str]
        if isinstance(self.value, Unset):
            value = UNSET
        elif self.value is None:
            value = None
        else:
            value = self.value

        numeric_value = self.numeric_value
        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if value is not UNSET:
            field_dict["value"] = value
        if numeric_value is not UNSET:
            field_dict["numeric_value"] = numeric_value
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        def _parse_value(data: object) -> Union[None, Unset, int, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int, str], data)

        value = _parse_value(d.pop("value", UNSET))

        numeric_value = d.pop("numeric_value", UNSET)

        enabled = d.pop("enabled", UNSET)

        module_access_level = cls(
            name=name,
            value=value,
            numeric_value=numeric_value,
            enabled=enabled,
        )

        module_access_level.additional_properties = d
        return module_access_level

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.basic_system_information import BasicSystemInformation
from ..models.channel_info import ChannelInfo
from ..models.config_statistics import ConfigStatistics
from ..models.memory_information import MemoryInformation
from ..models.misc_system_information import MiscSystemInformation
from ..models.system_stats import SystemStats
from ..types import UNSET, Unset

T = TypeVar("T", bound="SystemInformation")


@attr.s(auto_attribs=True)
class SystemInformation:
    """ """

    basic: Union[Unset, BasicSystemInformation] = UNSET
    memory: Union[Unset, MemoryInformation] = UNSET
    misc: Union[Unset, MiscSystemInformation] = UNSET
    config: Union[Unset, ConfigStatistics] = UNSET
    stats: Union[Unset, SystemStats] = UNSET
    channels: Union[Unset, List[ChannelInfo]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        basic: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.basic, Unset):
            basic = self.basic.to_dict()

        memory: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.memory, Unset):
            memory = self.memory.to_dict()

        misc: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.misc, Unset):
            misc = self.misc.to_dict()

        config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.config, Unset):
            config = self.config.to_dict()

        stats: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stats, Unset):
            stats = self.stats.to_dict()

        channels: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.channels, Unset):
            channels = []
            for channels_item_data in self.channels:
                channels_item = channels_item_data.to_dict()

                channels.append(channels_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if basic is not UNSET:
            field_dict["basic"] = basic
        if memory is not UNSET:
            field_dict["memory"] = memory
        if misc is not UNSET:
            field_dict["misc"] = misc
        if config is not UNSET:
            field_dict["config"] = config
        if stats is not UNSET:
            field_dict["stats"] = stats
        if channels is not UNSET:
            field_dict["channels"] = channels

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _basic = d.pop("basic", UNSET)
        basic: Union[Unset, BasicSystemInformation]
        if isinstance(_basic, Unset):
            basic = UNSET
        else:
            basic = BasicSystemInformation.from_dict(_basic)

        _memory = d.pop("memory", UNSET)
        memory: Union[Unset, MemoryInformation]
        if isinstance(_memory, Unset):
            memory = UNSET
        else:
            memory = MemoryInformation.from_dict(_memory)

        _misc = d.pop("misc", UNSET)
        misc: Union[Unset, MiscSystemInformation]
        if isinstance(_misc, Unset):
            misc = UNSET
        else:
            misc = MiscSystemInformation.from_dict(_misc)

        _config = d.pop("config", UNSET)
        config: Union[Unset, ConfigStatistics]
        if isinstance(_config, Unset):
            config = UNSET
        else:
            config = ConfigStatistics.from_dict(_config)

        _stats = d.pop("stats", UNSET)
        stats: Union[Unset, SystemStats]
        if isinstance(_stats, Unset):
            stats = UNSET
        else:
            stats = SystemStats.from_dict(_stats)

        channels = []
        _channels = d.pop("channels", UNSET)
        for channels_item_data in _channels or []:
            channels_item = ChannelInfo.from_dict(channels_item_data)

            channels.append(channels_item)

        system_information = cls(
            basic=basic,
            memory=memory,
            misc=misc,
            config=config,
            stats=stats,
            channels=channels,
        )

        system_information.additional_properties = d
        return system_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional

import httpx

from ...client import AuthenticatedClient
from ...models.new_news import NewNews
from ...models.news import News
from ...types import Response


def _get_kwargs(
    id: int,
    *,
    client: AuthenticatedClient,
    json_body: NewNews,
) -> Dict[str, Any]:
    url = "{}/news/{id}".format(client.base_url, id=id)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[News]:
    if response.status_code == 200:
        response_200 = News.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[News]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    json_body: NewNews,
) -> Response[News]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
    )

    response = client.client.patch(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
    json_body: NewNews,
) -> Optional[News]:
    """Modify an existing news item"""

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    json_body: NewNews,
) -> Response[News]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
    )

    response = await client.async_client.patch(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
    json_body: NewNews,
) -> Optional[News]:
    """Modify an existing news item"""

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ModuleSubcommandChannel")


@attr.s(auto_attribs=True)
class ModuleSubcommandChannel:
    """ """

    access_level: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_level = self.access_level
        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if access_level is not UNSET:
            field_dict["access_level"] = access_level
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_level = d.pop("access_level", UNSET)

        enabled = d.pop("enabled", UNSET)

        module_subcommand_channel = cls(
            access_level=access_level,
            enabled=enabled,
        )

        module_subcommand_channel.additional_properties = d
        return module_subcommand_channel

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RouteHopColor")


@attr.s(auto_attribs=True)
class RouteHopColor:
    """ """

    hop: Union[Unset, str] = UNSET
    where: Union[Unset, None, str] = UNSET
    via: Union[Unset, None, str] = UNSET
    tag_color: Union[Unset, None, str] = UNSET
    text_color: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        hop = self.hop
        where = self.where
        via = self.via
        tag_color = self.tag_color
        text_color = self.text_color

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if hop is not UNSET:
            field_dict["hop"] = hop
        if where is not UNSET:
            field_dict["where"] = where
        if via is not UNSET:
            field_dict["via"] = via
        if tag_color is not UNSET:
            field_dict["tag_color"] = tag_color
        if text_color is not UNSET:
            field_dict["text_color"] = text_color

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        hop = d.pop("hop", UNSET)

        where = d.pop("where", UNSET)

        via = d.pop("via", UNSET)

        tag_color = d.pop("tag_color", UNSET)

        text_color = d.pop("text_color", UNSET)

        route_hop_color = cls(
            hop=hop,
            where=where,
            via=via,
            tag_color=tag_color,
            text_color=text_color,
        )

        route_hop_color.additional_properties = d
        return route_hop_color

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="OrgHistory")


@attr.s(auto_attribs=True)
class OrgHistory:
    """ """

    id: Union[Unset, int] = UNSET
    actor: Union[Unset, None, str] = UNSET
    actee: Union[Unset, None, str] = UNSET
    action: Union[Unset, None, str] = UNSET
    organization: Union[Unset, None, str] = UNSET
    time: Union[Unset, None, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        actor = self.actor
        actee = self.actee
        action = self.action
        organization = self.organization
        time = self.time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if actor is not UNSET:
            field_dict["actor"] = actor
        if actee is not UNSET:
            field_dict["actee"] = actee
        if action is not UNSET:
            field_dict["action"] = action
        if organization is not UNSET:
            field_dict["organization"] = organization
        if time is not UNSET:
            field_dict["time"] = time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        actor = d.pop("actor", UNSET)

        actee = d.pop("actee", UNSET)

        action = d.pop("action", UNSET)

        organization = d.pop("organization", UNSET)

        time = d.pop("time", UNSET)

        org_history = cls(
            id=id,
            actor=actor,
            actee=actee,
            action=action,
            organization=organization,
            time=time,
        )

        org_history.additional_properties = d
        return org_history

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.module_subcommand import ModuleSubcommand
from ..models.module_subcommand_channel import ModuleSubcommandChannel
from ..types import UNSET, Unset

T = TypeVar("T", bound="ModuleCommand")


@attr.s(auto_attribs=True)
class ModuleCommand:
    """ """

    command: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    msg: Union[Unset, ModuleSubcommandChannel] = UNSET
    priv: Union[Unset, ModuleSubcommandChannel] = UNSET
    org: Union[Unset, ModuleSubcommandChannel] = UNSET
    subcommands: Union[Unset, List[ModuleSubcommand]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        command = self.command
        type = self.type
        description = self.description
        msg: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.msg, Unset):
            msg = self.msg.to_dict()

        priv: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.priv, Unset):
            priv = self.priv.to_dict()

        org: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.org, Unset):
            org = self.org.to_dict()

        subcommands: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.subcommands, Unset):
            subcommands = []
            for subcommands_item_data in self.subcommands:
                subcommands_item = subcommands_item_data.to_dict()

                subcommands.append(subcommands_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if command is not UNSET:
            field_dict["command"] = command
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if msg is not UNSET:
            field_dict["msg"] = msg
        if priv is not UNSET:
            field_dict["priv"] = priv
        if org is not UNSET:
            field_dict["org"] = org
        if subcommands is not UNSET:
            field_dict["subcommands"] = subcommands

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        command = d.pop("command", UNSET)

        type = d.pop("type", UNSET)

        description = d.pop("description", UNSET)

        _msg = d.pop("msg", UNSET)
        msg: Union[Unset, ModuleSubcommandChannel]
        if isinstance(_msg, Unset):
            msg = UNSET
        else:
            msg = ModuleSubcommandChannel.from_dict(_msg)

        _priv = d.pop("priv", UNSET)
        priv: Union[Unset, ModuleSubcommandChannel]
        if isinstance(_priv, Unset):
            priv = UNSET
        else:
            priv = ModuleSubcommandChannel.from_dict(_priv)

        _org = d.pop("org", UNSET)
        org: Union[Unset, ModuleSubcommandChannel]
        if isinstance(_org, Unset):
            org = UNSET
        else:
            org = ModuleSubcommandChannel.from_dict(_org)

        subcommands = []
        _subcommands = d.pop("subcommands", UNSET)
        for subcommands_item_data in _subcommands or []:
            subcommands_item = ModuleSubcommand.from_dict(subcommands_item_data)

            subcommands.append(subcommands_item)

        module_command = cls(
            command=command,
            type=type,
            description=description,
            msg=msg,
            priv=priv,
            org=org,
            subcommands=subcommands,
        )

        module_command.additional_properties = d
        return module_command

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Player")


@attr.s(auto_attribs=True)
class Player:
    """This represents the data the bot stores about a player in the cache and database"""

    charid: Union[Unset, int] = UNSET
    first_name: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    level: Union[Unset, None, int] = UNSET
    breed: Union[Unset, str] = UNSET
    gender: Union[Unset, str] = UNSET
    faction: Union[Unset, str] = UNSET
    profession: Union[Unset, None, str] = UNSET
    prof_title: Union[Unset, str] = UNSET
    ai_rank: Union[Unset, str] = UNSET
    ai_level: Union[Unset, None, int] = UNSET
    org_id: Union[Unset, None, int] = UNSET
    org: Union[Unset, None, str] = UNSET
    org_rank: Union[Unset, None, str] = UNSET
    org_rank_id: Union[Unset, None, int] = UNSET
    dimension: Union[Unset, None, int] = UNSET
    head_id: Union[Unset, None, int] = UNSET
    pvp_rating: Union[Unset, None, int] = UNSET
    pvp_title: Union[Unset, None, str] = UNSET
    last_update: Union[Unset, None, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        charid = self.charid
        first_name = self.first_name
        name = self.name
        last_name = self.last_name
        level = self.level
        breed = self.breed
        gender = self.gender
        faction = self.faction
        profession = self.profession
        prof_title = self.prof_title
        ai_rank = self.ai_rank
        ai_level = self.ai_level
        org_id = self.org_id
        org = self.org
        org_rank = self.org_rank
        org_rank_id = self.org_rank_id
        dimension = self.dimension
        head_id = self.head_id
        pvp_rating = self.pvp_rating
        pvp_title = self.pvp_title
        last_update = self.last_update

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if charid is not UNSET:
            field_dict["charid"] = charid
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if name is not UNSET:
            field_dict["name"] = name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if level is not UNSET:
            field_dict["level"] = level
        if breed is not UNSET:
            field_dict["breed"] = breed
        if gender is not UNSET:
            field_dict["gender"] = gender
        if faction is not UNSET:
            field_dict["faction"] = faction
        if profession is not UNSET:
            field_dict["profession"] = profession
        if prof_title is not UNSET:
            field_dict["prof_title"] = prof_title
        if ai_rank is not UNSET:
            field_dict["ai_rank"] = ai_rank
        if ai_level is not UNSET:
            field_dict["ai_level"] = ai_level
        if org_id is not UNSET:
            field_dict["org_id"] = org_id
        if org is not UNSET:
            field_dict["org"] = org
        if org_rank is not UNSET:
            field_dict["org_rank"] = org_rank
        if org_rank_id is not UNSET:
            field_dict["org_rank_id"] = org_rank_id
        if dimension is not UNSET:
            field_dict["dimension"] = dimension
        if head_id is not UNSET:
            field_dict["head_id"] = head_id
        if pvp_rating is not UNSET:
            field_dict["pvp_rating"] = pvp_rating
        if pvp_title is not UNSET:
            field_dict["pvp_title"] = pvp_title
        if last_update is not UNSET:
            field_dict["last_update"] = last_update

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        charid = d.pop("charid", UNSET)

        first_name = d.pop("first_name", UNSET)

        name = d.pop("name", UNSET)

        last_name = d.pop("last_name", UNSET)

        level = d.pop("level", UNSET)

        breed = d.pop("breed", UNSET)

        gender = d.pop("gender", UNSET)

        faction = d.pop("faction", UNSET)

        profession = d.pop("profession", UNSET)

        prof_title = d.pop("prof_title", UNSET)

        ai_rank = d.pop("ai_rank", UNSET)

        ai_level = d.pop("ai_level", UNSET)

        org_id = d.pop("org_id", UNSET)

        org = d.pop("org", UNSET)

        org_rank = d.pop("org_rank", UNSET)

        org_rank_id = d.pop("org_rank_id", UNSET)

        dimension = d.pop("dimension", UNSET)

        head_id = d.pop("head_id", UNSET)

        pvp_rating = d.pop("pvp_rating", UNSET)

        pvp_title = d.pop("pvp_title", UNSET)

        last_update = d.pop("last_update", UNSET)

        player = cls(
            charid=charid,
            first_name=first_name,
            name=name,
            last_name=last_name,
            level=level,
            breed=breed,
            gender=gender,
            faction=faction,
            profession=profession,
            prof_title=prof_title,
            ai_rank=ai_rank,
            ai_level=ai_level,
            org_id=org_id,
            org=org,
            org_rank=org_rank,
            org_rank_id=org_rank_id,
            dimension=dimension,
            head_id=head_id,
            pvp_rating=pvp_rating,
            pvp_title=pvp_title,
            last_update=last_update,
        )

        player.additional_properties = d
        return player

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NewNews")


@attr.s(auto_attribs=True)
class NewNews:
    """ """

    time: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    news: Union[Unset, str] = UNSET
    sticky: Union[Unset, bool] = UNSET
    deleted: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        time = self.time
        name = self.name
        news = self.news
        sticky = self.sticky
        deleted = self.deleted

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if time is not UNSET:
            field_dict["time"] = time
        if name is not UNSET:
            field_dict["name"] = name
        if news is not UNSET:
            field_dict["news"] = news
        if sticky is not UNSET:
            field_dict["sticky"] = sticky
        if deleted is not UNSET:
            field_dict["deleted"] = deleted

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        time = d.pop("time", UNSET)

        name = d.pop("name", UNSET)

        news = d.pop("news", UNSET)

        sticky = d.pop("sticky", UNSET)

        deleted = d.pop("deleted", UNSET)

        new_news = cls(
            time=time,
            name=name,
            news=news,
            sticky=sticky,
            deleted=deleted,
        )

        new_news.additional_properties = d
        return new_news

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.org_history import OrgHistory
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    actor: Union[Unset, None, str] = UNSET,
    actee: Union[Unset, None, str] = UNSET,
    action: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, int] = UNSET,
    after: Union[Unset, None, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/org/history".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "limit": limit,
        "actor": actor,
        "actee": actee,
        "action": action,
        "before": before,
        "after": after,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[OrgHistory]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = OrgHistory.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[OrgHistory]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    actor: Union[Unset, None, str] = UNSET,
    actee: Union[Unset, None, str] = UNSET,
    action: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, int] = UNSET,
    after: Union[Unset, None, int] = UNSET,
) -> Response[List[OrgHistory]]:
    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        actor=actor,
        actee=actee,
        action=action,
        before=before,
        after=after,
    )

    response = client.client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    actor: Union[Unset, None, str] = UNSET,
    actee: Union[Unset, None, str] = UNSET,
    action: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, int] = UNSET,
    after: Union[Unset, None, int] = UNSET,
) -> Optional[List[OrgHistory]]:
    """Query entries from the org history log"""

    return sync_detailed(
        client=client,
        limit=limit,
        actor=actor,
        actee=actee,
        action=action,
        before=before,
        after=after,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    actor: Union[Unset, None, str] = UNSET,
    actee: Union[Unset, None, str] = UNSET,
    action: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, int] = UNSET,
    after: Union[Unset, None, int] = UNSET,
) -> Response[List[OrgHistory]]:
    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        actor=actor,
        actee=actee,
        action=action,
        before=before,
        after=after,
    )

    response = await client.async_client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    actor: Union[Unset, None, str] = UNSET,
    actee: Union[Unset, None, str] = UNSET,
    action: Union[Unset, None, str] = UNSET,
    before: Union[Unset, None, int] = UNSET,
    after: Union[Unset, None, int] = UNSET,
) -> Optional[List[OrgHistory]]:
    """Query entries from the org history log"""

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            actor=actor,
            actee=actee,
            action=action,
            before=before,
            after=after,
        )
    ).parsed



""" Contains all the data models used in inputs/outputs """

from .audit import Audit
from .basic_system_information import BasicSystemInformation
from .channel_info import ChannelInfo
from .class_spec import ClassSpec
from .config_module import ConfigModule
from .config_statistics import ConfigStatistics
from .function_parameter import FunctionParameter
from .memory_information import MemoryInformation
from .misc_system_information import MiscSystemInformation
from .module_access_level import ModuleAccessLevel
from .module_command import ModuleCommand
from .module_event_config import ModuleEventConfig
from .module_setting import ModuleSetting
from .module_subcommand import ModuleSubcommand
from .module_subcommand_channel import ModuleSubcommandChannel
from .new_news import NewNews
from .news import News
from .online_player import OnlinePlayer
from .online_players import OnlinePlayers
from .operation import Operation
from .org_history import OrgHistory
from .player import Player
from .proxy_capabilities import ProxyCapabilities
from .proxy_reply import ProxyReply
from .relay_config import RelayConfig
from .relay_layer import RelayLayer
from .relay_layer_argument import RelayLayerArgument
from .relay_status import RelayStatus
from .route_hop_color import RouteHopColor
from .route_hop_format import RouteHopFormat
from .setting_option import SettingOption
from .system_information import SystemInformation
from .system_stats import SystemStats
